/******************************************************************************
 THXApp.cc

	BASE CLASS = JXApplication

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXApp.h"
#include "THXVarList.h"
#include "THXVarDirector.h"
#include "THXExprDirector.h"
#include "THXPlotDirector.h"
#include "THXPlotFunctionDialog.h"
#include "THXBaseConvDirector.h"
#include "THXAboutDialog.h"
#include "thxGlobals.h"
#include "thxFileVersions.h"
#include <JXHelpManager.h>
#include <JXWindow.h>
#include <JXTextMenu.h>
#include <JXPSPrinter.h>
#include <JXChooseSaveFile.h>
#include <JThisProcess.h>
#include <JString.h>
#include <jFileUtil.h>
#include <jDirUtil.h>
#include <fstream.h>
#include <jAssert.h>

extern const JCharacter* kTHXVersionNumberStr;

// application signature (MDI)

const JCharacter* THXApp::kAppSignature = "THX-1138";

// state information

const JCharacter* kStateFileName = ".THX-1138.session";

/******************************************************************************
 Constructor

 ******************************************************************************/

THXApp::THXApp
	(
	int*	argc,
	char*	argv[]
	)
	:
	JXApplication(argc, argv)
{
	itsStatePath = new JString;
	assert( itsStatePath != NULL );

	itsExprList = new JPtrArray<THXExprDirector>;
	assert( itsExprList != NULL );

	itsKeyPadVisibleFlag = kTrue;

	itsPlotList = new JPtrArray<THXPlotDirector>;
	assert( itsPlotList != NULL );
	itsPlotList->SetCompareFunction(ComparePlotTitles);
	itsPlotList->SetSortOrder(JOrderedSetT::kSortAscending);

	itsPlotFnDialog = NULL;

	THXCreateGlobals(this, kAppSignature);
	RestoreProgramState();
}

/******************************************************************************
 Destructor

	Delete global objects in reverse order of creation.

 ******************************************************************************/

THXApp::~THXApp()
{
	delete itsStatePath;
	delete itsVarList;
	delete itsExprList;		// objects deleted by JXDirector
	delete itsPlotList;		// objects deleted by JXDirector

	THXDeleteGlobals();
}

/******************************************************************************
 Close (virtual protected)

 ******************************************************************************/

JBoolean
THXApp::Close()
{
	if (!itsVarDirector->OKToDeactivate())
		{
		return kFalse;
		}

	SaveProgramState();
	return JXApplication::Close();
}

/******************************************************************************
 DisplayAbout

 ******************************************************************************/

void
THXApp::DisplayAbout
	(
	const JCharacter* prevVersStr
	)
{
	THXAboutDialog* dlog = new THXAboutDialog(this, prevVersStr);
	assert( dlog != NULL );
	dlog->BeginDialog();
}

/******************************************************************************
 NewExpression

 ******************************************************************************/

THXExprDirector*
THXApp::NewExpression
	(
	const JBoolean centerOnScreen
	)
{
	THXExprDirector* expr = new THXExprDirector(this, itsVarList);
	assert( expr != NULL );
	if (centerOnScreen)
		{
		(expr->GetWindow())->PlaceAsDialogWindow();
		}
	expr->Activate();
	itsExprList->Append(expr);
	return expr;
}

/******************************************************************************
 SetKeyPadVisible

 ******************************************************************************/

void
THXApp::SetKeyPadVisible
	(
	const JBoolean visible
	)
{
	if (visible != itsKeyPadVisibleFlag)
		{
		itsKeyPadVisibleFlag = visible;

		const JSize count = itsExprList->GetElementCount();
		for (JIndex i=1; i<=count; i++)
			{
			(itsExprList->NthElement(i))->UpdateDisplay();
			}
		}
}

/******************************************************************************
 NewPlot

	prevPlot should be the THXPlotDirector that originated the request,
	NULL otherwise.

 ******************************************************************************/

void
THXApp::NewPlot
	(
	const THXPlotDirector* prevPlot
	)
{
	assert( itsPlotFnDialog == NULL );

	itsPlotFnDialog = new THXPlotFunctionDialog(this, itsVarList, prevPlot);
	assert( itsPlotFnDialog != NULL );
	itsPlotFnDialog->Activate();
	ListenTo(itsPlotFnDialog);
}

/******************************************************************************
 CreatePlot (private)

 ******************************************************************************/

void
THXApp::CreatePlot()
{
	assert( itsPlotFnDialog != NULL );

	JIndex plotIndex;
	const JFunction* f;
	JString curveName;
	JFloat xMin, xMax;
	itsPlotFnDialog->GetSettings(&plotIndex, &f, &curveName, &xMin, &xMax);

	if (plotIndex > itsPlotList->GetElementCount())
		{
		THXPlotDirector* plot = new THXPlotDirector(this);
		assert( plot != NULL );
		itsPlotList->Append(plot);
		}

	THXPlotDirector* plot = itsPlotList->NthElement(plotIndex);
	plot->AddFunction(itsVarList, *f, curveName, xMin, xMax);
	plot->Activate();
}

/******************************************************************************
 ShowConstants

 ******************************************************************************/

void
THXApp::ShowConstants()
	const
{
	itsVarDirector->Activate();
}

/******************************************************************************
 ShowBaseConversion

 ******************************************************************************/

void
THXApp::ShowBaseConversion()
	const
{
	itsBCDirector->Activate();
}

/******************************************************************************
 RestoreProgramState (private)

 ******************************************************************************/

void
THXApp::RestoreProgramState()
{
JIndex i;

	if (!JGetHomeDirectory(itsStatePath))
		{
		(JGetUserNotification())->ReportError(
			"I am unable to find your home directory.");
		JThisProcess::Exit(1);
		}

	const JString fullName = JCombinePathAndName(*itsStatePath, kStateFileName);
	if (!JFileExists(fullName))
		{
		InitProgramState();
		DisplayAbout();
		return;
		}

	ifstream input(fullName);

	JFileVersion vers;
	input >> vers;
	if (input.eof() || input.fail())
		{
		InitProgramState();
		return;
		}
	else if (vers > kCurrentStateVersion)
		{
		JString msg = "Unable to read the file \"";
		msg += fullName;
		msg += "\" because it was created by a newer version of this program.";
		(JGetUserNotification())->ReportError(msg);
		JThisProcess::Exit(1);
		}

	JString prevProgramVers;
	input >> prevProgramVers;

	itsVarList = new THXVarList(input, vers);
	assert( itsVarList != NULL );

	itsVarDirector = new THXVarDirector(input, vers, this, itsVarList);
	assert( itsVarDirector != NULL );

	itsBCDirector = new THXBaseConvDirector(input, vers, this);
	assert( itsBCDirector != NULL );

	if (vers >= 1)
		{
		input >> itsKeyPadVisibleFlag;
		}

	if (vers >= 2)
		{
		THXExprDirector::ReadPrefs(input, vers);
		(JXGetHelpManager())->ReadSetup(input);
		}

	if (vers >= 5)
		{
		(JXGetChooseSaveFile())->ReadSetup(input);
		}

	if (vers >= 3)
		{
		(THXGetGraphPrinter())->ReadXPSSetup(input);
		}

	JSize exprCount;
	input >> exprCount;

	if (exprCount == 0)
		{
		NewExpression(kTrue);
		}
	else
		{
		for (i=1; i<=exprCount; i++)
			{
			THXExprDirector* expr = new THXExprDirector(input, vers, this, itsVarList);
			assert( expr != NULL );
			expr->Activate();
			itsExprList->Append(expr);

			if (exprCount == 1)
				{
				(expr->GetWindow())->Deiconify();
				}
			}
		}

	JSize plotCount;
	input >> plotCount;

	for (i=1; i<=plotCount; i++)
		{
		THXPlotDirector* plot = new THXPlotDirector(input, vers, this, itsVarList);
		assert( plot != NULL );
		plot->Activate();
		itsPlotList->Append(plot);
		}

	if (prevProgramVers != kTHXVersionNumberStr)
		{
		DisplayAbout(prevProgramVers);
		}
}

/******************************************************************************
 InitProgramState (private)

 ******************************************************************************/

void
THXApp::InitProgramState()
{
	itsVarList = new THXVarList;
	assert( itsVarList != NULL );

	itsVarDirector = new THXVarDirector(this, itsVarList);
	assert( itsVarDirector != NULL );

	itsBCDirector = new THXBaseConvDirector(this);
	assert( itsBCDirector != NULL );

	NewExpression(kTrue);
}

/******************************************************************************
 SaveProgramState (private)

 ******************************************************************************/

void
THXApp::SaveProgramState()
{
JIndex i;

	const JString fullName = JCombinePathAndName(*itsStatePath, kStateFileName);
	ofstream output(fullName);

	output << kCurrentStateVersion;
	output << ' ' << JString(kTHXVersionNumberStr);
	output << ' ' << *itsVarList;

	output << ' ';
	itsVarDirector->WriteState(output);

	output << ' ';
	itsBCDirector->WriteState(output);

	output << ' ' << itsKeyPadVisibleFlag;

	output << ' ';
	THXExprDirector::WritePrefs(output);

	output << ' ';
	(JXGetHelpManager())->WriteSetup(output);

	output << ' ';
	(JXGetChooseSaveFile())->WriteSetup(output);

	output << ' ';
	(THXGetGraphPrinter())->WriteXPSSetup(output);

	const JSize exprCount = itsExprList->GetElementCount();
	output << ' ' << exprCount;

	for (i=1; i<=exprCount; i++)
		{
		output << ' ';
		(itsExprList->NthElement(i))->WriteState(output);
		}

	const JSize plotCount = itsPlotList->GetElementCount();
	output << ' ' << plotCount;

	for (i=1; i<=plotCount; i++)
		{
		output << ' ';
		(itsPlotList->NthElement(i))->WriteState(output);
		}
}

/******************************************************************************
 Receive (protected)

 ******************************************************************************/

void
THXApp::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsPlotFnDialog && message.Is(JXDialogDirector::kDeactivated))
		{
		const JXDialogDirector::Deactivated* info =
			dynamic_cast(const JXDialogDirector::Deactivated*, &message);
		assert( info != NULL );
		if (info->Successful())
			{
			CreatePlot();
			}
		itsPlotFnDialog = NULL;
		}

	else
		{
		JXApplication::Receive(sender, message);
		}
}

/*****************************************************************************
 DirectorClosed (virtual protected)

	Listen for windows that are closed.

 ******************************************************************************/

void
THXApp::DirectorClosed
	(
	JXDirector* theDirector
	)
{
	JIndex dirIndex;
	THXExprDirector* exprDir = dynamic_cast(THXExprDirector*, theDirector);
	if (exprDir != NULL && itsExprList->Find(exprDir, &dirIndex))
		{
		itsExprList->RemoveElement(dirIndex);
		}
	THXPlotDirector* plotDir = dynamic_cast(THXPlotDirector*, theDirector);
	if (plotDir != NULL && itsPlotList->Find(plotDir, &dirIndex))
		{
		itsPlotList->RemoveElement(dirIndex);
		}

	JXApplication::DirectorClosed(theDirector);

	if (itsExprList->IsEmpty())
		{
		Quit();
		}
}

/*****************************************************************************
 BuildPlotMenu

	If origPlot != NULL, initialChoice is set to the index of the director.

	We are deactivated until a plot is chosen, so sorting itsPlotList
	is safe.

 ******************************************************************************/

void
THXApp::BuildPlotMenu
	(
	JXTextMenu*				menu,
	const THXPlotDirector*	origPlot,
	JIndex*					initialChoice
	)
	const
{
	itsPlotList->Sort();

	*initialChoice = 0;

	const JSize count = itsPlotList->GetElementCount();
	for (JIndex i=1; i<=count; i++)
		{
		const THXPlotDirector* plot = itsPlotList->NthElement(i);
		menu->AppendItem((plot->GetWindow())->GetTitle(), kTrue, kTrue);

		if (plot == origPlot)
			{
			*initialChoice = i;
			}
		}

	if (*initialChoice == 0)
		{
		*initialChoice = count+1;
		}

	if (count > 0)
		{
		menu->ShowSeparatorAfter(count);
		}
	menu->AppendItem("New window", kTrue, kTrue);
	menu->SetUpdateAction(JXMenu::kDisableNone);
}

/******************************************************************************
 ComparePlotTitles (static private)

 ******************************************************************************/

JOrderedSetT::CompareResult
THXApp::ComparePlotTitles
	(
	THXPlotDirector* const & p1,
	THXPlotDirector* const & p2
	)
{
	return JCompareStringsCaseInsensitive(
			const_cast<JString*>(&((p1->GetWindow())->GetTitle())),
			const_cast<JString*>(&((p2->GetWindow())->GetTitle())));
}

/******************************************************************************
 CleanUpBeforeSuddenDeath (virtual protected)

	*** If the server is dead, you cannot call any code that contacts it.

 ******************************************************************************/

void
THXApp::CleanUpBeforeSuddenDeath
	(
	const JXDocumentManager::SafetySaveReason reason
	)
{
	JXApplication::CleanUpBeforeSuddenDeath(reason);

	if (reason != JXDocumentManager::kAssertFired)
		{
		SaveProgramState();
		}
}

#include <JArray.tmpls>
#define JTemplateType THXExprDirector
#include <JPtrArray.tmpls>
#undef JTemplateType

#include <JArray.tmpls>
#define JTemplateType THXPlotDirector
#include <JPtrArray.tmpls>
#undef JTemplateType
