/******************************************************************************
 THXBaseConvDirector.cc

	Window to convert non-negative integers from one base to another.

	BASE CLASS = JXWindowDirector

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXBaseConvDirector.h"
#include "THXBaseConvMenu.h"
#include "thxGlobals.h"
#include "thxPixmaps.h"
#include "thxHelpText.h"
#include <JXHelpManager.h>
#include <JXDisplay.h>
#include <JXWindow.h>
#include <JXTextButton.h>
#include <JXStaticText.h>
#include <JXInputField.h>
#include <JRegex.h>
#include <JString.h>
#include <jStrStreamUtil.h>
#include <iomanip.h>
#include <jAssert.h>

/******************************************************************************
 Constructor

 ******************************************************************************/

THXBaseConvDirector::THXBaseConvDirector
	(
	JXDirector* supervisor
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow();
}

THXBaseConvDirector::THXBaseConvDirector
	(
	istream&			input,
	const JFileVersion	vers,
	JXDirector*			supervisor
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow();

	JSize fromBase, toBase;
	input >> fromBase >> toBase;
	itsFromBase->SetBase(fromBase);
	itsToBase->SetBase(toBase);

	JString fromValue;
	input >> fromValue;
	itsFromValue->SetText(fromValue);

	JXWindow* window = GetWindow();
	window->ReadGeometry(input);
	window->Deiconify();

	JBoolean active;
	input >> active;
	if (active)
		{
		Activate();
		}
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXBaseConvDirector::~THXBaseConvDirector()
{
	delete itsWindowIcon;
}

/******************************************************************************
 WriteState

 ******************************************************************************/

void
THXBaseConvDirector::WriteState
	(
	ostream& output
	)
	const
{
	output << itsFromBase->GetBase();
	output << ' ' << itsToBase->GetBase();
	output << ' ' << itsFromValue->GetText();

	output << ' ';
	(GetWindow())->WriteGeometry(output);

	output << ' ' << JI2B(IsActive() && !(GetWindow())->IsIconified());
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
THXBaseConvDirector::BuildWindow()
{
// begin JXLayout

    JXWindow* window = new JXWindow(this, 300,130, "");
    assert( window != NULL );
    SetWindow(window);

    itsCloseButton =
        new JXTextButton("Close", window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 180,100, 60,20);
    assert( itsCloseButton != NULL );
    itsCloseButton->SetShortcuts("#W^[");

    itsFromValue =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kFixedTop, 110,20, 70,20);
    assert( itsFromValue != NULL );

    JXStaticText* obj1 =
        new JXStaticText("Convert from:", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,20, 90,20);
    assert( obj1 != NULL );

    itsHelpButton =
        new JXTextButton("Help", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 60,100, 60,20);
    assert( itsHelpButton != NULL );

    itsFromBase =
        new THXBaseConvMenu(window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 190,15, 90,30);
    assert( itsFromBase != NULL );

    JXStaticText* obj2 =
        new JXStaticText("Result:", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,60, 90,20);
    assert( obj2 != NULL );

    itsToBase =
        new THXBaseConvMenu(window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 190,55, 90,30);
    assert( itsToBase != NULL );

    itsToValue =
        new JXStaticText("", kFalse, kTrue, NULL, window,
                    JXWidget::kHElastic, JXWidget::kFixedTop, 110,60, 70,20);
    assert( itsToValue != NULL );

// end JXLayout

	window->SetTitle("Base Conversion");
	window->LockCurrentMinSize();
	window->SetCloseAction(JXWindow::kDeactivateDirector);
	window->SetFocusWhenShow(kTrue);
	window->PlaceAsDialogWindow();

	JXDisplay* display = GetDisplay();
	itsWindowIcon = new JXImage(display, display->GetColormap(), THXGetBaseConvWindowIcon());
	assert( itsWindowIcon != NULL );
	window->SetIcon(*itsWindowIcon);

	ListenTo(itsFromValue);
	ListenTo(itsFromBase);
	ListenTo(itsToBase);
	ListenTo(itsCloseButton);
	ListenTo(itsHelpButton);
}

/******************************************************************************
 Receive (virtual protected)

 ******************************************************************************/

void
THXBaseConvDirector::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsCloseButton && message.Is(JXButton::kPushed))
		{
		(GetWindow())->Close();
		}
	else if (sender == itsHelpButton && message.Is(JXButton::kPushed))
		{
		(JXGetHelpManager())->ShowSection(kTHXBaseConvHelpName);
		}

	else if (sender == itsFromValue &&
			 (message.Is(JTextEditor::kTextSet) ||
			  message.Is(JTextEditor::kTextChanged)))
		{
		Convert();
		}
	else if ((sender == itsFromBase || sender == itsToBase) &&
			 message.Is(THXBaseConvMenu::kBaseChanged))
		{
		Convert();
		}

	else
		{
		JXWindowDirector::Receive(sender, message);
		}
}

/******************************************************************************
 Convert (private)

 ******************************************************************************/

static const JRegex hexRegex = "^[[:space:]]*0[xX]";

void
THXBaseConvDirector::Convert()
{
	itsFromValue->ClearUndo();		// keep receiving TextChanged

	const JInteger fromBase = itsFromBase->GetBase();

	const JString& fromStr = itsFromValue->GetText();
	if (fromBase != 16 && hexRegex.Match(fromStr))
		{
		itsFromBase->SetBase(16);	// calls us again
		return;
		}

	JUInt value;
	if (!fromStr.ConvertToUInt(&value, fromBase))
		{
		itsToValue->SetText("");
		return;
		}

	JString toValue;
	const JInteger toBase = itsToBase->GetBase();
	if (value == 0)
		{
		toValue = "0";
		}
	else if (toBase == 2)
		{
		do
			{
			if (value & 0x01)
				{
				toValue.PrependCharacter('1');
				}
			else
				{
				toValue.PrependCharacter('0');
				}
			value = value >> 1;
			}
			while (value != 0);
		}
	else
		{
		ostrstream s;
		s << setbase(toBase) << value << ends;
		toValue = s.str();
		JUnfreeze(s);
		}

	if (toBase == 16)
		{
		toValue.ToUpper();
		toValue.Prepend("0x");
		}

	itsToValue->SetText(toValue);
	itsToValue->SelectAll();
}
