/******************************************************************************
 THXExprDirector.cc

	Window to display expression being edited by user.

	BASE CLASS = JXWindowDirector

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXExprDirector.h"
#include "THXExprEditor.h"
#include "THXKeyPad.h"
#include "THXVarList.h"
#include "thxGlobals.h"
#include "thxPixmaps.h"
#include "thxHelpText.h"
#include "thxFileVersions.h"
#include <JXDisplay.h>
#include <JXWindow.h>
#include <JXStaticText.h>
#include <JXMenuBar.h>
#include <JXTextMenu.h>
#include <JXTextButton.h>
#include <JXVertPartition.h>
#include <JXScrollbarSet.h>
#include <JXScrollbar.h>
#include <JXHelpManager.h>
#include <JXChooseSaveFile.h>
#include <jXActionDefs.h>
#include <JFunction.h>
#include <fstream.h>
#include <jStrStreamUtil.h>
#include <jAssert.h>

JString	THXExprDirector::itsDefGeom;
const JFileVersion kCurrentDefGeomVersion = 0;

// Actions menu

static const JCharacter* kActionsMenuTitleStr = "Actions";
static const JCharacter* kActionsMenuStr =
	"    New expression     %k Meta-N"
	"  | Edit constants     %k Meta-E"
	"  | Plot function...   %k Meta-Shift-P"
	"  | Base conversion... %k Meta-B"
	"%l| Page setup..."
	"  | Print tape...      %k Meta-P"
	"%l| Save tape...       %k Meta-S"
	"  | Clear tape"
	"%l| Quit               %k Meta-Q";

enum
{
	kNewExprCmd = 1,
	kEditConstCmd,
	kNewPlotCmd,
	kConvBaseCmd,
	kPageSetupCmd, kPrintTapeCmd,
	kSaveTapeCmd, kClearTapeCmd,
	kQuitCmd
};

// Preferences menu

static const JCharacter* kPrefsMenuTitleStr = "Preferences";
static const JCharacter* kPrefsMenuStr =
	"Show keypad %b %l | Save window size";

enum
{
	kToggleKeyPadVisibleCmd = 1,
	kSaveWindSizeCmd
};

// Help menu

static const JCharacter* kHelpMenuTitleStr = "Help";
static const JCharacter* kHelpMenuStr =
	"    About" 
	"%l| Table of Contents %i" kJXHelpTOCAction
	"  | Overview"
	"  | This window       %i" kJXHelpSpecificAction
	"%l| Changes"
	"  | Credits";

enum
{
	kHelpAboutCmd = 1,
	kHelpTOCCmd, kHelpOverviewCmd, kHelpWindowCmd,
	kHelpChangeLogCmd, kHelpCreditsCmd
};

/******************************************************************************
 Constructor

 ******************************************************************************/

THXExprDirector::THXExprDirector
	(
	JXDirector*			supervisor,
	const THXVarList*	varList
	)
	:
	JXWindowDirector(supervisor),
	itsTapeName("Untitled")
{
	BuildWindow(varList);
	UseDefaultGeometry();
}

THXExprDirector::THXExprDirector
	(
	istream&			input,
	const JFileVersion	vers,
	JXDirector*			supervisor,
	const THXVarList*	varList
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow(varList);

	JFunction* f = JFunction::StreamIn(input, varList, kTrue);
	itsExprWidget->SetFunction(varList, f);

	JString tapeText;
	input >> tapeText >> itsTapeName;
	itsTapeWidget->SetText(tapeText);

	if (vers >= 4)
		{
		JString printName;
		input >> printName;
		itsTapeWidget->SetPTPrintFileName(printName);
		}

	(GetWindow())->ReadGeometry(input);
	itsPartition->ReadGeometry(input);
	itsExprWidget->ReadScrollSetup(input);
	itsTapeWidget->ReadScrollSetup(input);
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXExprDirector::~THXExprDirector()
{
	delete itsWindowIcon;
}

/******************************************************************************
 WriteState

 ******************************************************************************/

void
THXExprDirector::WriteState
	(
	ostream& output
	)
	const
{
	const JFunction* f = itsExprWidget->GetFunction();
	f->StreamOut(output);

	output << ' ' << itsTapeWidget->GetText();
	output << ' ' << itsTapeName;
	output << ' ' << itsTapeWidget->GetPTPrintFileName();

	output << ' ';
	(GetWindow())->WriteGeometry(output);

	output << ' ';
	itsPartition->WriteGeometry(output);

	output << ' ';
	itsExprWidget->WriteScrollSetup(output);

	output << ' ';
	itsTapeWidget->WriteScrollSetup(output);
}

/******************************************************************************
 ReadPrefs (static)

 ******************************************************************************/

void
THXExprDirector::ReadPrefs
	(
	istream&			input,
	const JFileVersion	vers
	)
{
	if (vers >= 2)
		{
		input >> itsDefGeom;
		}
}

/******************************************************************************
 WritePrefs (static)

 ******************************************************************************/

void
THXExprDirector::WritePrefs
	(
	ostream& output
	)
{
	output << itsDefGeom;
}

/******************************************************************************
 UseDefaultGeometry (private)

 ******************************************************************************/

void
THXExprDirector::UseDefaultGeometry()
{
	if (!itsDefGeom.IsEmpty())
		{
		jistrstream(input, itsDefGeom.GetCString(), itsDefGeom.GetLength());

		JFileVersion vers;
		input >> vers;
		if (vers <= kCurrentDefGeomVersion)
			{
			JCoordinate w,h;
			input >> w >> h;
			(GetWindow())->SetSize(w,h);

			itsPartition->ReadGeometry(input);
			}
		}
}

/******************************************************************************
 SetDefaultGeometry (private)

 ******************************************************************************/

void
THXExprDirector::SetDefaultGeometry()
{
	ostrstream data;
	data << kCurrentDefGeomVersion;
	data << ' ' << (GetWindow())->GetFrameWidth();
	data << ' ' << (GetWindow())->GetFrameHeight();
	data << ' ';
	itsPartition->WriteGeometry(data);

	itsDefGeom = data.str();
	JUnfreeze(data);
}

/******************************************************************************
 Activate (virtual)

 ******************************************************************************/

void
THXExprDirector::Activate()
{
	JXWindowDirector::Activate();
	itsExprWidget->Focus();
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
THXExprDirector::BuildWindow
	(
	const THXVarList* varList
	)
{
	JArray<JCoordinate> heights(2);
	heights.AppendElement(100);
	heights.AppendElement(0);

	const JIndex elasticIndex = 2;

	JArray<JCoordinate> minHeights(2);
	minHeights.AppendElement(50);
	minHeights.AppendElement(50);

// begin JXLayout

    JXWindow* window = new JXWindow(this, 360,240, "");
    assert( window != NULL );
    SetWindow(window);

    JXMenuBar* menuBar =
        new JXMenuBar(window,
                    JXWidget::kHElastic, JXWidget::kFixedTop, 0,0, 310,30);
    assert( menuBar != NULL );

    itsPartition =
        new JXVertPartition(heights, elasticIndex, minHeights, window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 0,30, 300,210);
    assert( itsPartition != NULL );

    itsCloseButton =
        new JXTextButton("Close", window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 310,0, 50,30);
    assert( itsCloseButton != NULL );
    itsCloseButton->SetShortcuts("#W");

    itsKeyPad =
        new THXKeyPad(window,
                    JXWidget::kFixedRight, JXWidget::kVElastic, 300,30, 60,210);
    assert( itsKeyPad != NULL );

// end JXLayout

	window->SetTitle("THX-1138");
	window->LockCurrentMinSize();
	window->SetFocusWhenShow(kTrue);
	window->SetWMClass(THXGetWMClassInstance(), THXGetExprWindowClass());

	JXDisplay* display = GetDisplay();
	itsWindowIcon = new JXImage(display, display->GetColormap(), THXGetExprWindowIcon());
	assert( itsWindowIcon != NULL );
	window->SetIcon(*itsWindowIcon);

	ListenTo(itsCloseButton);

	// create tape

	JXScrollbarSet* scrollbarSet2 =
		new JXScrollbarSet(itsPartition->GetCompartment(2),
						   JXWidget::kHElastic, JXWidget::kVElastic,
						   0,0, 100,100);
	assert( scrollbarSet2 != NULL );
	scrollbarSet2->FitToEnclosure();

	itsTapeWidget =
		new JXStaticText("", kTrue, kTrue, scrollbarSet2,
						 scrollbarSet2->GetScrollEnclosure(),
						 JXWidget::kHElastic, JXWidget::kVElastic,
						 0,0, 10,10);
	assert( itsTapeWidget != NULL );
	itsTapeWidget->FitToEnclosure();
	itsTapeWidget->SetBackColor(itsTapeWidget->GetFocusColor());
	itsTapeWidget->SetPTPrinter(THXGetTapePrinter());

	// create expr editor -- requires tape widget

	JXScrollbarSet* scrollbarSet1 =
		new JXScrollbarSet(itsPartition->GetCompartment(1),
						   JXWidget::kHElastic, JXWidget::kVElastic,
						   0,0, 100,100);
	assert( scrollbarSet1 != NULL );
	scrollbarSet1->FitToEnclosure();

	itsExprWidget =
		new THXExprEditor(varList, menuBar, itsTapeWidget, scrollbarSet1,
						  scrollbarSet1->GetScrollEnclosure(),
						  JXWidget::kHElastic, JXWidget::kVElastic,
						  0,0, 10,10);
	assert( itsExprWidget != NULL );
	itsExprWidget->FitToEnclosure();
	itsExprWidget->SetBackColor(itsExprWidget->GetFocusColor());

	itsKeyPad->SetExprEditor(itsExprWidget);
	itsTapeWidget->ShareEditMenu(itsExprWidget->GetEditMenu(),
								 JXMenu::kMacintoshStyle, kFalse, kFalse);

	// create menus

	itsActionsMenu = menuBar->PrependTextMenu(kActionsMenuTitleStr);
	itsActionsMenu->SetMenuItems(kActionsMenuStr);
	itsActionsMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsActionsMenu);

	itsPrefsMenu = menuBar->AppendTextMenu(kPrefsMenuTitleStr);
	itsPrefsMenu->SetMenuItems(kPrefsMenuStr);
	itsPrefsMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsPrefsMenu);

	itsHelpMenu = menuBar->AppendTextMenu(kHelpMenuTitleStr);
	itsHelpMenu->SetMenuItems(kHelpMenuStr);
	itsHelpMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsHelpMenu);

	// keypad visibility

	UpdateDisplay();
}

/******************************************************************************
 UpdateDisplay

 ******************************************************************************/

void
THXExprDirector::UpdateDisplay()
{
	if (itsKeyPad->WouldBeVisible() && !(THXGetApplication())->KeyPadIsVisible())
		{
		itsKeyPad->Hide();
		itsPartition->AdjustSize(itsKeyPad->GetFrameWidth(), 0);
		}
	else if (!itsKeyPad->WouldBeVisible() && (THXGetApplication())->KeyPadIsVisible())
		{
		itsKeyPad->Show();
		itsPartition->AdjustSize(-itsKeyPad->GetFrameWidth(), 0);
		}
}

/******************************************************************************
 Receive (virtual protected)

 ******************************************************************************/

void
THXExprDirector::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsCloseButton && message.Is(JXButton::kPushed))
		{
		(GetWindow())->Close();
		}

	else if (sender == itsActionsMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdateActionsMenu();
		}
	else if (sender == itsActionsMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleActionsMenu(selection->GetIndex());
		}

	else if (sender == itsPrefsMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdatePrefsMenu();
		}
	else if (sender == itsPrefsMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandlePrefsMenu(selection->GetIndex());
		}

	else if (sender == itsHelpMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdateHelpMenu();
		}
	else if (sender == itsHelpMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleHelpMenu(selection->GetIndex());
		}

	else
		{
		JXWindowDirector::Receive(sender, message);
		}
}

/******************************************************************************
 UpdateActionsMenu (private)

 ******************************************************************************/

void
THXExprDirector::UpdateActionsMenu()
{
}

/******************************************************************************
 HandleActionsMenu (private)

 ******************************************************************************/

void
THXExprDirector::HandleActionsMenu
	(
	const JIndex index
	)
{
	if (index == kNewExprCmd)
		{
		(THXGetApplication())->NewExpression();
		}
	else if (index == kEditConstCmd)
		{
		(THXGetApplication())->ShowConstants();
		}
	else if (index == kNewPlotCmd)
		{
		(THXGetApplication())->NewPlot();
		}
	else if (index == kConvBaseCmd)
		{
		(THXGetApplication())->ShowBaseConversion();
		}

	else if (index == kPageSetupCmd)
		{
		itsTapeWidget->HandlePTPageSetup();
		}
	else if (index == kPrintTapeCmd)
		{
		itsTapeWidget->PrintPT();
		}

	else if (index == kSaveTapeCmd)
		{
		SaveTape();
		}
	else if (index == kClearTapeCmd)
		{
		// this way, it is undoable

		itsTapeWidget->SelectAll();
		itsTapeWidget->DeleteSelection();
		}

	else if (index == kQuitCmd)
		{
		(THXGetApplication())->Quit();
		}
}

/******************************************************************************
 SaveTape (private)

 ******************************************************************************/

void
THXExprDirector::SaveTape()
{
	JString newName;
	if ((JXGetChooseSaveFile())->SaveFile("Save tape as:", NULL,
										  itsTapeName, &newName))
		{
		itsTapeName = newName;

		ofstream output(itsTapeName);
		(itsTapeWidget->GetText()).Print(output);
		}
}

/******************************************************************************
 UpdatePrefsMenu (private)

 ******************************************************************************/

void
THXExprDirector::UpdatePrefsMenu()
{
	if ((THXGetApplication())->KeyPadIsVisible())
		{
		itsPrefsMenu->CheckItem(kToggleKeyPadVisibleCmd);
		}
}

/******************************************************************************
 HandlePrefsMenu (private)

 ******************************************************************************/

void
THXExprDirector::HandlePrefsMenu
	(
	const JIndex index
	)
{
	if (index == kToggleKeyPadVisibleCmd)
		{
		(THXGetApplication())->ToggleKeyPadVisible();
		}

	else if (index == kSaveWindSizeCmd)
		{
		SetDefaultGeometry();
		}
}

/******************************************************************************
 UpdateHelpMenu (private)

 ******************************************************************************/

void
THXExprDirector::UpdateHelpMenu()
{
}

/******************************************************************************
 HandleHelpMenu (private)

 ******************************************************************************/

void
THXExprDirector::HandleHelpMenu
	(
	const JIndex index
	)
{
	if (index == kHelpAboutCmd)
		{
		(THXGetApplication())->DisplayAbout();
		}

	else if (index == kHelpTOCCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXTOCHelpName);
		}
	else if (index == kHelpOverviewCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXOverviewHelpName);
		}
	else if (index == kHelpWindowCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXExprHelpName);
		}

	else if (index == kHelpChangeLogCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXChangeLogName);
		}
	else if (index == kHelpCreditsCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXCreditsName);
		}
}
