/******************************************************************************
 THXPlotDirector.cc

	BASE CLASS = JXWindowDirector

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXPlotDirector.h"
#include "THXVarList.h"
#include "thxGlobals.h"
#include "thxPixmaps.h"
#include "thxHelpText.h"
#include <JXDisplay.h>
#include <JXWindow.h>
#include <JXTextButton.h>
#include <JXMenuBar.h>
#include <JXTextMenu.h>
#include <JXHelpManager.h>
#include <jXActionDefs.h>
#include <JX2DPlotWidget.h>
#include <JPlotJFunction.h>
#include <JFunction.h>
#include <JString.h>
#include <jAssert.h>

// Actions menu

static const JCharacter* kActionsMenuTitleStr = "Actions";
static const JCharacter* kActionsMenuStr =
	"    New expression     %k Meta-N"
	"  | Edit constants     %k Meta-E"
	"  | Plot function...   %k Meta-Shift-P"
	"  | Base conversion... %k Meta-B"
	"%l| Page setup..."
	"  | Print...           %k Meta-P"
	"%l| Quit               %k Meta-Q";

enum
{
	kNewExprCmd = 1,
	kEditConstCmd,
	kNewPlotCmd,
	kConvBaseCmd,
	kPSPageSetupCmd, kPrintPSCmd,
	kQuitCmd
};

// Help menu

static const JCharacter* kHelpMenuTitleStr = "Help";
static const JCharacter* kHelpMenuStr =
	"    About" 
	"%l| Table of Contents %i" kJXHelpTOCAction
	"  | Overview"
	"  | This window       %i" kJXHelpSpecificAction
	"%l| Changes"
	"  | Credits";

enum
{
	kHelpAboutCmd = 1,
	kHelpTOCCmd, kHelpOverviewCmd, kHelpWindowCmd,
	kHelpChangeLogCmd, kHelpCreditsCmd
};

/******************************************************************************
 Constructor

 ******************************************************************************/

THXPlotDirector::THXPlotDirector
	(
	JXDirector*	supervisor
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow();
}

THXPlotDirector::THXPlotDirector
	(
	istream&			input,
	const JFileVersion	vers,
	JXDirector*			supervisor,
	THXVarList*			varList
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow();
	(GetWindow())->ReadGeometry(input);

	JSize fnCount;
	input >> fnCount;

	for (JIndex i=1; i<=fnCount; i++)
		{
		JFloat xMin, xMax;
		input >> xMin >> xMax;

		JFunction* f = JFunction::StreamIn(input, varList);
		AddFunction(varList, f, "", xMin, xMax);
		}

	itsPlotWidget->PWXReadSetup(input);
	itsPlotWidget->PWReadCurveSetup(input);

	if (4 <= vers && vers <= 5)
		{
		JString s;
		input >> s;
		itsPlotWidget->SetPSPrintFileName(s);
		}
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXPlotDirector::~THXPlotDirector()
{
	delete itsWindowIcon;
}

/******************************************************************************
 WriteState

 ******************************************************************************/

void
THXPlotDirector::WriteState
	(
	ostream& output
	)
	const
{
	(GetWindow())->WriteGeometry(output);

	const JSize fnCount = itsPlotWidget->GetCurveCount();
	output << ' ' << fnCount;

	for (JIndex i=1; i<=fnCount; i++)
		{
		const JPlotDataBase& data    = itsPlotWidget->GetCurve(i);
		const JPlotJFunction* fnData = dynamic_cast(const JPlotJFunction*, &data);
		assert( fnData != NULL );

		JFloat xMin, xMax;
		fnData->GetXRange(&xMin, &xMax);
		output << ' ' << xMin << ' ' << xMax;

		output << ' ';
		(fnData->GetFunction()).StreamOut(output);
		}

	output << ' ';
	itsPlotWidget->PWXWriteSetup(output);

	output << ' ';
	itsPlotWidget->PWWriteCurveSetup(output);
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
THXPlotDirector::BuildWindow()
{
// begin JXLayout

    JXWindow* window = new JXWindow(this, 600,400, "");
    assert( window != NULL );
    SetWindow(window);

    JXMenuBar* menuBar =
        new JXMenuBar(window,
                    JXWidget::kHElastic, JXWidget::kFixedTop, 0,0, 550,30);
    assert( menuBar != NULL );

    itsPlotWidget =
        new JX2DPlotWidget(menuBar, window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 0,30, 600,370);
    assert( itsPlotWidget != NULL );

    itsCloseButton =
        new JXTextButton("Close", window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 550,0, 50,30);
    assert( itsCloseButton != NULL );
    itsCloseButton->SetShortcuts("#W");

// end JXLayout

	window->SetMinSize(300, 200);
	window->SetWMClass(THXGetWMClassInstance(), THXGetPlotWindowClass());

	JXDisplay* display = GetDisplay();
	itsWindowIcon = new JXImage(display, display->GetColormap(), THXGetPlotWindowIcon());
	assert( itsWindowIcon != NULL );
	window->SetIcon(*itsWindowIcon);

	itsPlotWidget->SetPSPrinter(THXGetGraphPrinter());

	ListenTo(itsPlotWidget);
	ListenTo(itsCloseButton);

	itsActionsMenu = menuBar->PrependTextMenu(kActionsMenuTitleStr);
	itsActionsMenu->SetMenuItems(kActionsMenuStr);
	itsActionsMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsActionsMenu);

	itsHelpMenu = menuBar->AppendTextMenu(kHelpMenuTitleStr);
	itsHelpMenu->SetMenuItems(kHelpMenuStr);
	itsHelpMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsHelpMenu);

	// do this after everything is constructed so Receive() doesn't crash

	itsPlotWidget->SetTitle("New plot");
}

/******************************************************************************
 AddFunction

 ******************************************************************************/

void
THXPlotDirector::AddFunction
	(
	THXVarList*			varList,
	const JFunction&	f,
	const JCharacter*	name,
	const JFloat		xMin,
	const JFloat		xMax
	)
{
	AddFunction(varList, f.Copy(), name, xMin, xMax);
}

/******************************************************************************
 AddFunction (private)

	We take ownership of the function.

 ******************************************************************************/

void
THXPlotDirector::AddFunction
	(
	THXVarList*			varList,
	JFunction*			f,
	const JCharacter*	name,
	const JFloat		xMin,
	const JFloat		xMax
	)
{
	JPlotJFunction* data =
		new JPlotJFunction(itsPlotWidget, varList, f, kTrue,
						   THXVarList::kXIndex, xMin, xMax);
	assert( data != NULL );

	itsPlotWidget->AddCurve(data, kTrue, name, kTrue, kFalse);
}

/******************************************************************************
 Receive (virtual protected)

 ******************************************************************************/

void
THXPlotDirector::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsPlotWidget && message.Is(J2DPlotWidget::kTitleChanged))
		{
		(GetWindow())->SetTitle(itsPlotWidget->GetTitle());
		}

	else if (sender == itsCloseButton && message.Is(JXButton::kPushed))
		{
		(GetWindow())->Close();
		}

	else if (sender == itsActionsMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdateActionsMenu();
		}
	else if (sender == itsActionsMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleActionsMenu(selection->GetIndex());
		}

	else if (sender == itsHelpMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdateHelpMenu();
		}
	else if (sender == itsHelpMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleHelpMenu(selection->GetIndex());
		}

	else
		{
		JXWindowDirector::Receive(sender, message);
		}
}

/******************************************************************************
 UpdateActionsMenu (private)

 ******************************************************************************/

void
THXPlotDirector::UpdateActionsMenu()
{
}

/******************************************************************************
 HandleActionsMenu (private)

 ******************************************************************************/

void
THXPlotDirector::HandleActionsMenu
	(
	const JIndex index
	)
{
	if (index == kNewExprCmd)
		{
		(THXGetApplication())->NewExpression();
		}
	else if (index == kEditConstCmd)
		{
		(THXGetApplication())->ShowConstants();
		}
	else if (index == kNewPlotCmd)
		{
		(THXGetApplication())->NewPlot(this);
		}
	else if (index == kConvBaseCmd)
		{
		(THXGetApplication())->ShowBaseConversion();
		}

	else if (index == kPSPageSetupCmd)
		{
		itsPlotWidget->HandlePSPageSetup();
		}
	else if (index == kPrintPSCmd)
		{
		itsPlotWidget->PrintPS();
		}

	else if (index == kQuitCmd)
		{
		(THXGetApplication())->Quit();
		}
}

/******************************************************************************
 UpdateHelpMenu (private)

 ******************************************************************************/

void
THXPlotDirector::UpdateHelpMenu()
{
}

/******************************************************************************
 HandleHelpMenu (private)

 ******************************************************************************/

void
THXPlotDirector::HandleHelpMenu
	(
	const JIndex index
	)
{
	if (index == kHelpAboutCmd)
		{
		(THXGetApplication())->DisplayAbout();
		}

	else if (index == kHelpTOCCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXTOCHelpName);
		}
	else if (index == kHelpOverviewCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXOverviewHelpName);
		}
	else if (index == kHelpWindowCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXPlotHelpName);
		}

	else if (index == kHelpChangeLogCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXChangeLogName);
		}
	else if (index == kHelpCreditsCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXCreditsName);
		}
}
