/******************************************************************************
 THXVarDirector.cc

	Window to display list of constants defined by user.

	BASE CLASS = JXWindowDirector

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXVarDirector.h"
#include "THXVarList.h"
#include "THXVarTable.h"
#include "thxGlobals.h"
#include "thxHelpText.h"
#include <JXHelpManager.h>
#include <JXWindow.h>
#include <JXTextButton.h>
#include <JXMenuBar.h>
#include <JXTextMenu.h>
#include <JXExprInput.h>
#include <JXScrollbarSet.h>
#include <JXScrollbar.h>
#include <JXColHeaderWidget.h>
#include <jXActionDefs.h>
#include <JString.h>
#include <jAssert.h>

// Actions menu

static const JCharacter* kActionsMenuTitleStr = "Actions";
static const JCharacter* kActionsMenuStr =
	"    New constant       %k Meta-Shift-N"
	"  | Remove constant    %k Meta-R"
	"%l| New expression     %k Meta-N"
	"  | Plot function...   %k Meta-Shift-P"
	"  | Base conversion... %k Meta-B"
	"%l| Quit               %k Meta-Q";

enum
{
	kNewConstCmd = 1,
	kRemoveConstCmd,
	kNewExprCmd,
	kNewPlotCmd,
	kConvBaseCmd,
	kQuitCmd
};

// Help menu

static const JCharacter* kHelpMenuTitleStr = "Help";
static const JCharacter* kHelpMenuStr =
	"    About" 
	"%l| Table of Contents %i" kJXHelpTOCAction
	"  | Overview"
	"  | This window       %i" kJXHelpSpecificAction
	"%l| Changes"
	"  | Credits";

enum
{
	kHelpAboutCmd = 1,
	kHelpTOCCmd, kHelpOverviewCmd, kHelpWindowCmd,
	kHelpChangeLogCmd, kHelpCreditsCmd
};

/******************************************************************************
 Constructor

 ******************************************************************************/

THXVarDirector::THXVarDirector
	(
	JXDirector*	supervisor,
	THXVarList*	varList
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow(varList);
}

THXVarDirector::THXVarDirector
	(
	istream&			input,
	const JFileVersion	vers,
	JXDirector*			supervisor,
	THXVarList*			varList
	)
	:
	JXWindowDirector(supervisor)
{
	BuildWindow(varList);

	JXWindow* window = GetWindow();
	window->ReadGeometry(input);
	window->Deiconify();

	itsVarTable->ReadScrollSetup(input);

	JSize colWidth;
	input >> colWidth;
	itsVarTable->SetColWidth(THXVarTable::kNameColumn, colWidth);

	JBoolean active;
	input >> active;
	if (active)
		{
		Activate();
		}
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXVarDirector::~THXVarDirector()
{
}

/******************************************************************************
 WriteState

 ******************************************************************************/

void
THXVarDirector::WriteState
	(
	ostream& output
	)
	const
{
	(GetWindow())->WriteGeometry(output);

	output << ' ';
	itsVarTable->WriteScrollSetup(output);

	output << ' ' << itsVarTable->GetColWidth(THXVarTable::kNameColumn);
	output << ' ' << JI2B(IsActive() && !(GetWindow())->IsIconified());
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
THXVarDirector::BuildWindow
	(
	THXVarList* varList
	)
{
// begin JXLayout

    JXWindow* window = new JXWindow(this, 260,250, "");
    assert( window != NULL );
    SetWindow(window);

    JXMenuBar* menuBar =
        new JXMenuBar(window,
                    JXWidget::kHElastic, JXWidget::kFixedTop, 0,0, 210,30);
    assert( menuBar != NULL );

    JXScrollbarSet* scrollbarSet =
        new JXScrollbarSet(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 0,50, 260,200);
    assert( scrollbarSet != NULL );

    itsCloseButton =
        new JXTextButton("Close", window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 210,0, 50,30);
    assert( itsCloseButton != NULL );
    itsCloseButton->SetShortcuts("#W^[");

    JXWidgetSet* colHdrContainer =
        new JXWidgetSet(window,
                    JXWidget::kHElastic, JXWidget::kFixedTop, 0,30, 260,20);
    assert( colHdrContainer != NULL );

// end JXLayout

	window->SetTitle("Constants");
	window->SetMinSize(150,150);
	window->SetCloseAction(JXWindow::kDeactivateDirector);
	window->SetFocusWhenShow(kTrue);

	ListenTo(itsCloseButton);

	itsActionsMenu = menuBar->AppendTextMenu(kActionsMenuTitleStr);
	itsActionsMenu->SetMenuItems(kActionsMenuStr);
	itsActionsMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsActionsMenu);

	JXTextMenu* fontMenu = JXExprInput::CreateFontMenu(menuBar, JXMenu::kMacintoshStyle);
	menuBar->AppendMenu(fontMenu);

	itsHelpMenu = menuBar->AppendTextMenu(kHelpMenuTitleStr);
	itsHelpMenu->SetMenuItems(kHelpMenuStr);
	itsHelpMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsHelpMenu);

	itsVarTable =
		new THXVarTable(varList, fontMenu, scrollbarSet,
						scrollbarSet->GetScrollEnclosure(),
						JXWidget::kHElastic, JXWidget::kVElastic,
						0,0, 10,10);
	assert( itsVarTable != NULL );

	JXColHeaderWidget* colHeader =
		JXColHeaderWidget::Create(itsVarTable, scrollbarSet, colHdrContainer,
							  JXWidget::kHElastic, JXWidget::kVElastic, 0,0, 10,10);
	assert( colHeader != NULL );
	colHeader->FitToEnclosure();
	colHeader->SetColTitle(1, "Name");
	colHeader->SetColTitle(2, "Value");
	colHeader->TurnOnColResizing(20);
}

/******************************************************************************
 Receive (virtual protected)

 ******************************************************************************/

void
THXVarDirector::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsCloseButton && message.Is(JXButton::kPushed))
		{
		(GetWindow())->Close();
		}

	else if (sender == itsActionsMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdateActionsMenu();
		}
	else if (sender == itsActionsMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleActionsMenu(selection->GetIndex());
		}

	else if (sender == itsHelpMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		UpdateHelpMenu();
		}
	else if (sender == itsHelpMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleHelpMenu(selection->GetIndex());
		}

	else
		{
		JXWindowDirector::Receive(sender, message);
		}
}

/******************************************************************************
 UpdateActionsMenu (private)

 ******************************************************************************/

void
THXVarDirector::UpdateActionsMenu()
{
	itsActionsMenu->SetItemEnable(kRemoveConstCmd,
								  itsVarTable->OKToRemoveSelectedConstant());
}

/******************************************************************************
 HandleActionsMenu (private)

 ******************************************************************************/

void
THXVarDirector::HandleActionsMenu
	(
	const JIndex index
	)
{
	if (index == kNewConstCmd)
		{
		itsVarTable->NewConstant();
		}
	else if (index == kRemoveConstCmd)
		{
		itsVarTable->RemoveSelectedConstant();
		}
	else if (index == kNewExprCmd)
		{
		(THXGetApplication())->NewExpression();
		}
	else if (index == kNewPlotCmd)
		{
		(THXGetApplication())->NewPlot();
		}
	else if (index == kConvBaseCmd)
		{
		(THXGetApplication())->ShowBaseConversion();
		}

	else if (index == kQuitCmd)
		{
		(THXGetApplication())->Quit();
		}
}

/******************************************************************************
 UpdateHelpMenu (private)

 ******************************************************************************/

void
THXVarDirector::UpdateHelpMenu()
{
}

/******************************************************************************
 HandleHelpMenu (private)

 ******************************************************************************/

void
THXVarDirector::HandleHelpMenu
	(
	const JIndex index
	)
{
	if (index == kHelpAboutCmd)
		{
		(THXGetApplication())->DisplayAbout();
		}

	else if (index == kHelpTOCCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXTOCHelpName);
		}
	else if (index == kHelpOverviewCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXOverviewHelpName);
		}
	else if (index == kHelpWindowCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXConstantsHelpName);
		}

	else if (index == kHelpChangeLogCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXChangeLogName);
		}
	else if (index == kHelpCreditsCmd)
		{
		(JXGetHelpManager())->ShowSection(kTHXCreditsName);
		}
}

/******************************************************************************
 OKToDeactivate (virtual)

	Check that the table contents are valid.

	This is public so THXApp can call it.

 ******************************************************************************/

JBoolean
THXVarDirector::OKToDeactivate()
{
	return JI2B( JXWindowDirector::OKToDeactivate() && itsVarTable->EndEditing() );
}
